﻿#include <Array.au3>

; https://www.autoitscript.com/wiki/ADO_Example_Excel

Global $aFiles[5] = [@ScriptDir & "\bsp1.xls", _
		@ScriptDir & "\bsp2.xls", _
		@ScriptDir & "\bsp3.xls", _
		@ScriptDir & "\bsp4.xls", _
		@ScriptDir & "\bsp5.xls"]


_Loop()


Func _Loop()
	For $i = 0 To UBound($aFiles) - 1
		Local $aExcelSheets = _XLSReadToArray($aFiles[$i])
		_ArrayDisplay($aExcelSheets, "returned: " & $aFiles[$i])
		For $ii = 0 To UBound($aExcelSheets) - 1
			_ArrayDisplay($aExcelSheets[$ii], "returned: " & $aFiles[$i] & " - Sheet: " & $ii + 1)
		Next
	Next
EndFunc   ;==>_Loop


Func _XLSReadToArray($sFullPath)

	Local Const $ADO_adSchemaTables = 20
	Local Const $iCursorType = 0
	Local Const $iLockType = 1
	Local Const $iOptions = 512

	; Eine Verbindung, zum Auslesen der Tabellennamen.
	Local $oADOConnection = ObjCreate("ADODB.Connection") ; Das ADODB-Verbindungsobjekt

	; Hier folgt die Verbindungszeichenkette. Infos dazu im ersten Link. Im zweiten Link Erklärung für den hier notwendigen Parameter IMEX=1.
	; https://support.microsoft.com/de-de/kb/278973
	; https://support.microsoft.com/de-de/kb/509360
	Local $sADOConnectionString = 'Provider=Microsoft.Jet.OLEDB.4.0' & _
								  ';Data Source=' & $sFullPath & _
								  ';Extended Properties="Excel 8.0' & _
								  ';HDR=NO' & _
								  ';IMEX=1"'

	$oADOConnection.Open($sADOConnectionString)


	Local $aSheetnames[0] ; Array, in dem die Tabellennamen gespeichert werden.

	; Beginn: Hier werden die Tabellen-Namen ausgelesen
	Local $oTablesSchema = $oADOConnection.OpenSchema($ADO_adSchemaTables)

	While Not $oTablesSchema.EOF
		_ArrayAdd($aSheetnames, $oTablesSchema.Fields("TABLE_NAME").Value)
		$oTablesSchema.MoveNext
	WEnd
	; Ende: Hier werden die Tabellen-Namen ausgelesen


	Local $iNrOfSheets = UBound($aSheetnames)

	Local $aExcelSheets[$iNrOfSheets] ; Das Rückgabe Array. In jedem Element soll ein Tabellenblatt gespeichert werden. Daher bekommt das Array die Ubound-Größe von $aSheetnames als Elementeanzahl


	; Nun werden in einer Schleife alle Tabellenblätter der Excel-Datei ausgelesen.
	For $i = 0 To $iNrOfSheets - 1

		$sSNReplace = StringReplace($aSheetnames[$i], "'", "")
		If StringRight($sSNReplace, 1) <> "$" Then ContinueLoop ; Zur Absturzvermeidung: Ist das letzte Zeichen kein $, kommt es zum Absturz nach dem Select Statement.

		Local $oADORecordset = ObjCreate("ADODB.Recordset") ; Das Rekordset-Objekt für das jeweilige Tabellenblatt

		Local $sSheet = "", $iSheetRows = 0

		Local $sADOSQL = "Select * FROM [" & $aSheetnames[$i] & "]" ; Hier wird in Schleife verwendet zeilenweise alles aus dem Tabellenblatt ausgelesen (Select *).
		$oADORecordset.Open($sADOSQL, $oADOConnection, $iCursorType, $iLockType, $iOptions)

		With $oADORecordset
			While Not .EOF
				For $oField In .Fields
					$sSheet &= $oField.Value & "|"
				Next
				$sSheet = StringTrimRight($sSheet, 1) & @LF
				$iSheetRows += 1
				.MoveNext
			WEnd
		EndWith

		Local $iSheetCols = 500
		Local $aSheets[$iSheetRows][$iSheetCols] ; Einfach ein großes Array, das alle Einträge der aktuellen Tabelle fassen können sollte. Dabei wird mal von 500 Spaten ausgegangen.

		; $sSheet ist jetzt ein String mit @LF Umbrüchen nach jeder Zeile und dem Verkettungszeichen | als Trenner zwischen den Spalten.
		; Das wird nun in einigen Umständlichen Operationen zu einem Array gewandelt.
		$sSheet = StringTrimRight($sSheet, 1)
		$aSheet = StringSplit($sSheet, @LF, 2)
		$iColCount = 0
		For $ii = 0 To $iSheetRows - 1
			$aColValues = StringSplit($aSheet[$ii], "|")
			If $aColValues[0] > $iSheetRows Then
				$iSheetCols = $aColValues[0]
				ReDim $aSheets[$iSheetRows][$iSheetCols] ; Je Zeile wird geprüft, ob das Array auch genügend Spalten hat, sonst erfolg ein ReDim auf die notwendige Anzahl.
			EndIf
			If $aColValues[0] > $iColCount Then $iColCount = $aColValues[0]
			For $iii = 1 To $aColValues[0]
				$aSheets[$ii][$iii - 1] = $aColValues[$iii]
			Next
		Next

		$oADORecordset.Close ; Recordset Objekt wird geschlossen.
		$oADORecordset = 0

		ReDim $aSheets[$iSheetRows][$iColCount] ; Das große Array wird auf die ermittelte, nötige Größe geschrumpft und in der folgenden Zeile zu einem $aExcelSheets-Element.
		$aExcelSheets[$i] = $aSheets
	Next
	$oADOConnection.Close ; Abschließend wird das Verbindungsobjekt Objekt wird geschlossen.
	$oADOConnection = 0

	Return $aExcelSheets
EndFunc   ;==>_XLSReadToArray
